'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.buildSite = undefined;

var _showdown = require('showdown');

var _glob = require('glob');

var _fs = require('fs');

var _lodash = require('lodash');

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _moment = require('moment');

var _moment2 = _interopRequireDefault(_moment);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/* global process */

var markdownConverter = new _showdown.Converter({
  simplifiedAutoLink: true,
  strikethrough: true,
  tables: true
});

/**
 * Recursively build the template, this allows for includes to contain includes …
 */
var buildTemplate = function buildTemplate(templateString, data, step) {
  step = step || 1;
  if (step >= 10) {
    console.error('Reached maximum nesting level', step);
    return templateString;
  }
  var previousResult = templateString;
  var result = (0, _lodash.template)(templateString)(data);
  if (result === previousResult) {
    return result;
  }
  return buildTemplate(result, data, ++step);
};

var isPage = function isPage(entry) {
  return entry.sys.contentType.sys.id === 'page';
};
var isPost = function isPost(entry) {
  return entry.sys.contentType.sys.id === 'post';
};
var isAuthor = function isAuthor(entry) {
  return entry.sys.contentType.sys.id === 'author';
};

var buildPostContent = function buildPostContent(post, locale) {
  var content = {};
  Object.keys(post.fields).forEach(function (k) {
    switch (k) {
      case 'content':
      case 'abstract':
        content[k] = markdownConverter.makeHtml(post.fields[k][locale]);
        break;
      case 'hero':
        content[k] = {
          title: post.fields[k][locale].fields.title[locale],
          file: post.fields[k][locale].fields.file[locale]
        };
        break;
      case 'author':
        content[k] = {
          name: post.fields[k][locale].fields.name[locale],
          slug: post.fields[k][locale].fields.slug[locale]
        };
        break;
      default:
        content[k] = post.fields[k][locale];
    }
  });
  return content;
};

var buildAuthorContent = function buildAuthorContent(author, locale) {
  var content = {};
  Object.keys(author.fields).forEach(function (k) {
    switch (k) {
      case 'description':
        content[k] = markdownConverter.makeHtml(author.fields[k][locale]);
        break;
      case 'photo':
        content[k] = {
          title: author.fields[k][locale].fields.title[locale],
          file: author.fields[k][locale].fields.file[locale]
        };
        break;
      default:
        content[k] = author.fields[k][locale];
    }
  });
  return content;
};

var buildPageContent = function buildPageContent(page, locale) {
  var content = {};
  Object.keys(page.fields).forEach(function (k) {
    switch (k) {
      case 'content':
        content[k] = markdownConverter.makeHtml(page.fields[k][locale]);
        break;
      default:
        content[k] = page.fields[k][locale];
    }
  });
  return content;
};

var shortDate = function shortDate(date) {
  return (0, _moment2.default)(date).format('DD.MM.YYYY');
};
var striptags = function striptags(str) {
  return str.replace(/<[^>]+>/g, '');
};

// This renders a page
var buildPage = function buildPage(build, config, collections, content, identifier, template, includesFiles) {
  var includes = {};
  var page = {
    url: config.webHost + config.baseHref + identifier + '.html'
  };
  var templateData = {
    build: Object.assign({ identifier: identifier }, build),
    config: config,
    content: content,
    collections: collections,
    includes: includes,
    shortDate: shortDate,
    striptags: striptags,
    page: page
  };

  // Build includes
  Object.keys(includesFiles).forEach(function (k) {
    includes[k] = buildTemplate(includesFiles[k], templateData);
  });

  // Build page
  var pageTemplate = buildTemplate((0, _fs.readFileSync)(template, 'utf8'), templateData);

  // Build pages
  (0, _fs.writeFileSync)('build/' + identifier + '.html', pageTemplate);

  // Done
  console.log('build/' + identifier + '.html');

  return templateData;
};

var buildSite = exports.buildSite = function buildSite(contentFile, templateDir, version, locale) {
  var environment = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : 'production';

  var content = JSON.parse((0, _fs.readFileSync)(contentFile, 'utf-8'));

  // Find includes
  var includesDir = _path2.default.join(templateDir, '/includes/');
  var includesFiles = {};
  (0, _glob.sync)(includesDir + '*.html').map(function (f) {
    includesFiles[f.replace(includesDir, '').replace(/\.html$/, '')] = (0, _fs.readFileSync)(f, 'utf8');
  });

  var build = {
    environment: environment,
    version: version,
    time: Date.now(),
    locale: locale
  };

  // Build the config
  var config = {};
  content.entries.filter(function (e) {
    return e.sys.contentType.sys.id === 'config';
  }).map(function (c) {
    var key = c.fields.key[build.locale];
    config[key] = process.env['CONFIG_' + key.toUpperCase()] || c.fields.value[build.locale];
  });

  // Build collections
  var collections = {};
  content.entries.filter(function (e) {
    return e.sys.contentType.sys.id === 'collection';
  }).map(function (c) {
    collections[c.fields.title[build.locale]] = c.fields.items[build.locale].map(function (e) {
      if (isPage(e)) return buildPageContent(e, build.locale);
      if (isPost(e)) return buildPostContent(e, build.locale);
    });
  });

  // Posts
  var posts = content.entries.filter(function (e) {
    return isPost(e);
  }).map(function (post) {
    var content = buildPostContent(post, build.locale);
    return buildPage(build, config, collections, content, content.slug, _path2.default.join(templateDir, '/post.html'), includesFiles);
  });

  // Authors
  content.entries.filter(function (e) {
    return isAuthor(e);
  }).map(function (page) {
    var content = buildAuthorContent(page, build.locale);
    buildPage(build, config, collections, content, content.slug, _path2.default.join(templateDir, '/author.html'), includesFiles);
  });

  // Pages
  content.entries.filter(function (e) {
    return isPage(e);
  }).map(function (page) {
    var content = buildPageContent(page, build.locale);
    buildPage(build, config, collections, content, content.slug, _path2.default.join(templateDir, '/page.html'), includesFiles);
  });

  // Index
  buildPage(build, config, collections, { posts: posts }, 'index', _path2.default.join(templateDir, '/index.html'), includesFiles);
};